#include <opencv/cv.h>
#include <opencv/highgui.h>
#include <stdio.h>
#include <string.h>

void detect_and_draw( IplImage* img, CvHaarClassifierCascade* cascade,CvMemStorage* storage ) {
	IplImage* gray = cvCreateImage( cvSize(img->width,img->height), 8, 1 );
	cvCvtColor( img, gray, CV_BGR2GRAY );

	cvClearMemStorage( storage );
	if( cascade )	{
		CvSeq* faces = cvHaarDetectObjects( gray, cascade, storage,1.1, 2, 0,cvSize(30, 30), cvSize(0, 0) );
		int i;
		for( i = 0; i<faces->total; i++ )		{
			CvRect* r = (CvRect*)cvGetSeqElem( faces, i );
			cvRectangle(img,cvPoint(r->x,r->y),cvPoint(r->x+r->width, r->y+r->height),cvScalar(0, 255, 0, 0),1, 8, 0);
		}
	}
	cvShowImage( "Display", img );
	cvReleaseImage( &gray );
}

int main( int argc, char** argv )
{
	CvCapture* cam = 0;
	CvMemStorage* storage = 0;
	CvHaarClassifierCascade* faceDetector = 0;

	faceDetector = (CvHaarClassifierCascade*)cvLoad( "haarcascade_frontalface_alt2.xml", 0, 0, 0 );
	if( !faceDetector )	{
		printf( "ERROR: Could not load classifier cascaden" );
		return -1;
	}

	storage = cvCreateMemStorage(0);
	cam = cvCaptureFromCAM(0);
	if(!cam) {
		printf("could not detect camera device!");
		return -1;
	}

	cvNamedWindow( "Display", 1 );
	IplImage *frame, *frame_copy = 0;
	while(1)	{
		if( !cvGrabFrame( cam ))
			break;
		frame = cvRetrieveFrame(cam,0);
		frame_copy = cvCreateImage( cvSize(frame->width,frame->height),IPL_DEPTH_8U, frame->nChannels );
		cvCopy( frame, frame_copy, 0 );
		detect_and_draw( frame_copy, faceDetector, storage );
		cvReleaseImage( &frame_copy );
		if( cvWaitKey( 10 ) >= 0 )
			break;
	}
	cvReleaseImage( &frame_copy );
	cvReleaseCapture( &cam );

	cvDestroyWindow("result");
	return 0;
}

